/*
 *	main.c
 *
 *	MIDI Player
 *
 *	* Sat Aug 30 23:09:00 JST 2003 Naoyuki Sawa
 *	- 1st [XB
 *	* Tue Sep 02 24:00:00 JST 2003 Naoyuki Sawa
 *	- MIDIhCoXVB
 */
#include "app.h"

/* I[gs[ggpꍇ́A
 * EQUICK_REPEAT_PADS`ĂB
 * EQUICK_REPEAT_PADSɍI[gs[gLɂPAD_*̑gݍ킹w肵ĂB
 * EQUICK_REPEAT_DELAYɍI[gs[gJn܂ł̎Ԃw肵ĂB
 * I[gs[ggpȂꍇ́A
 * EQUICK_REPEAT_PADS`ȂłB
 * EQUICK_REPEAT_PADS/DELAY̒`̗LɊւ炸AI[gs[g͍s܂B
 */
#define QUICK_REPEAT
#define QUICK_REPEAT_PADS	(PAD_LF | PAD_UP | PAD_RI | PAD_DN)
#define QUICK_REPEAT_DELAY	SEC(0.25)

extern const unsigned char INST[];	/* F`pMODt@C */

/****************************************************************************
 *	萔E^
 ****************************************************************************/

#define FPS		20			/* t[[g */
#define SEC(s)		((int)(FPS * (s)))	/* b->t[ϊ */

typedef struct _MIDIINFO {
	char fname[MAXFILENAME + 1];		/* t@C */
	int size;				/* t@CTCY */
} MIDIINFO;

#define MAX_DISP	13			/* Sɕ\łt@C */

/****************************************************************************
 *	ϐ
 ****************************************************************************/

unsigned char vbuff[DISP_X * DISP_Y];	/* zVRAM */
SURFACE surface;			/* zVRAM̃T[tFCX */
RENDER render;				/* zVRAMւ̕` */

int draw;				/* ĕ`tO */
int now;				/* O[o^C} */
int joy;				/* pbh */

MIDIINFO midi_info[MAXDIR];		/* MIDIt@CXg */
int midi_count;				/* MIDIt@C̐   */
int cursel;				/* ݂̑It@CCfNX */
int curtop;				/* ŏi\t@CCfNX */
int curplay = -1;			/* Đt@CCfNX */

/****************************************************************************
 *	֐錾
 ****************************************************************************/

void err_disp(const char* fmt, ...);
void app_init();
void app_exit();
void schedule();
void run();

/****************************************************************************
 *	main
 ****************************************************************************/

int
main()
{
	/* B */
	app_init();

	/* C[vB */
	run();

	return 0;
}

/****************************************************************************
 *	err_disp
 ****************************************************************************/

void
err_disp(const char* fmt, ...)
{
	int len, x, y;
	char str[100]; /* ꂮ炢Ώ[H */
	va_list ap;

	va_start(ap, fmt);
	len = vsprintf(str, fmt, ap);
	va_end(ap);

	x = (DISP_X - 5 * len) / 2;
	y = (DISP_Y - 10) / 2;
	memset(vbuff, 0, sizeof vbuff);
	render_string(&render, x, y, str, 0x80, 3);
	draw = 1;
	for(;;) {
		schedule();
		if(joy & (TRG_A | TRG_B | TRG_START | TRG_SELECT)) break;
	}
}

/****************************************************************************
 *	app_init
 ****************************************************************************/

void
app_init()
{
	int retval, len;
	FILEINFO fi;
	MIDIINFO* mi;

	/* ʓIȏB */
	pceLCDDispStop();
	pceLCDSetBuffer(vbuff);
	pcePadSetTrigMode(PP_MODE_SINGLE);
	pceLCDDispStart();

	/* `B */
	surface.w = DISP_X;
	surface.h = DISP_Y;
	surface.vbuff = vbuff;
	render_init(&render, &surface, 0, 0, 0, 0, 0, 0, 0);

	/* N[Abv֐o^B */
	atexit(app_exit);

	/* MIDIt@CT܂B */
	memset(midi_info, 0, sizeof midi_info);
	midi_count = 0;
	mi = &midi_info[0];
	retval = pceFileFindOpen(&fi);
	if(retval != 0) DIE();
	while(midi_count < MAXDIR) {
		retval = pceFileFindNext(&fi);
		if(!retval) break; /* t@CȂ */
		/* gq".mid"łȂ΁A΂܂B */
		len = strlen(fi.filename);
		if(len < 4) continue;
		if(strcasecmp(&fi.filename[len - 4], ".mid") != 0) continue;
		/* t@CƃTCYi[܂B */
		if(len > MAXFILENAME) DIE();
		memcpy(mi->fname, fi.filename, len);
		mi->size = fi.length;
		midi_count++;
		mi++;
	}
	pceFileFindClose(&fi);
	if(midi_count == 0) {
		err_disp("MIDIt@C܂");
		exit(0);
	}
}

/****************************************************************************
 *	app_exit
 ****************************************************************************/

void
app_exit()
{
}

/****************************************************************************
 *	schedule
 ****************************************************************************/

void
schedule()
{
#ifdef QUICK_REPEAT
	static unsigned char rep[8]; /* I[gs[gJE^ */
#endif /*QUICK_REPEAT*/

	int x, y;

	/* ɃX^oCɗȂ悤ɁB */
	pceAppActiveResponse(AAR_ACTIVE);

	/* drcʕ\XV̂߁AʍXVvȂĂAb1͋ʍXVB */
	if(now % SEC(1) == 0) {
		draw = 1;
	}

	/* ʍXVB */
	if(draw) {
		pceLCDTrans();
		draw = 0;
	}

	/* VXeB */
	yield();

	/* pbhԍXVB */
	joy = pcePadGet();

	/* O[o^C}XVB */
	now++;

#ifdef QUICK_REPEAT
	/* I[gs[gB */
	for(x = 0; x < 8; x++) {
		y = 1 << x; /* PAD_* */
		if(QUICK_REPEAT_PADS & y) {
			if(joy & y) {
				if(rep[x] != 0xff) rep[x]++;
				if(rep[x] >= QUICK_REPEAT_DELAY) joy |= y << 8; /* TRG_* */
			} else {
				rep[x] = 0;
			}
		}
	}
#endif QUICK_REPEAT
}

/****************************************************************************
 *	run
 ****************************************************************************/

MIDIMAP map = {
	{
		[0 ... 127] = 1
	}, {
		[0 ... 34] = 2,
		[35 ... 36] = 0x80 | 3,	/* oXh */
		[37] = 2,
		[38] = 0x80 | 2,	/* XlAh */
		[39] = 2,
		[40] = 0x80 | 2,	/* XlAh */
		[41 ... 127] = 2
	},
};

void
run()
{
	int retval, i, y, c;
	const void* romp;
	MIDIINFO* mi;

	goto L10; /* ͋` */
	for(;;) {
		schedule();

		/* pbh͂Ȃ΁A܂B */
		if(!joy) continue;

		/* IH */
		if(joy & TRG_SELECT) {
			exit(0);
		}

		/* t@CIύXH */
		if(joy & TRG_UP && cursel > 0             ) cursel--;
		if(joy & TRG_DN && cursel < midi_count - 1) cursel++;
		if(cursel < curtop                 ) curtop = cursel                 ;
		if(cursel > curtop + (MAX_DISP - 1)) curtop = cursel - (MAX_DISP - 1);

		/* ĐJnH */
		if(joy & TRG_A) {
			midi_stop(); /* t@C񒆂̃mCY}邽߁AIɒ~ */
			mi = &midi_info[cursel];
			romp = file_defrag(mi->fname);
			retval = midi_play(romp, INST, &map);
			if(retval == 0) {
				curplay = cursel;
			} else {
				err_disp("Đł܂ł");
				/* p */
			}
		}

		/* Đ~H */
		if(joy & TRG_B) {
			midi_stop();
			curplay = -1;
		}

L10:
		/* t@CXg\B */
		memset(vbuff, 0, sizeof vbuff);
		mi = &midi_info[0];
		for(i = 0; i < midi_count; i++) {
			y = 8 + (i - curtop) * 6;
			if(i == cursel) {
				if(i == curplay) {
					render_rectangle_fill(&render, 0, y - 1, DISP_X, 6 + 1, 1);
					c = 3;
				} else {
					render_rectangle_fill(&render, 0, y - 1, DISP_X, 6 + 1, 2);
					c = 0;
				}
			} else {
				if(i == curplay) {
					c = 3;
				} else {
					c = 2;
				}
			}
			render_string(&render, 4 + 4 *  1, y, mi->fname, 0x82, c);
			render_printf(&render, 4 + 4 * 15, y, 0x82, c, "%8d bytes", mi->size);
			mi++;
		}

		/* ^Cg\Biɂ͂ݏot@CXgɏ㏑j */
		render_rectangle_fill(&render, 0, 0, DISP_X, 6 + 1, 3);
		render_string(&render, 0, 1,
		//       0123456789012345678901234567890123
			"========== MIDI Player ===========",
			0x82, 0);

		/* ʓ]B */
		draw = 1;
	}
}
